# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.get_knowledge_base_list_response_model import GetKnowledgeBaseListResponseModel
from ...types.get_or_create_rag_index_request_model import GetOrCreateRagIndexRequestModel
from ...types.knowledge_base_content_search_response_model import KnowledgeBaseContentSearchResponseModel
from ...types.knowledge_base_document_type import KnowledgeBaseDocumentType
from ...types.knowledge_base_sort_by import KnowledgeBaseSortBy
from ...types.sort_direction import SortDirection
from .raw_client import AsyncRawKnowledgeBaseClient, RawKnowledgeBaseClient
from .types.knowledge_base_get_or_create_rag_indexes_response_value import (
    KnowledgeBaseGetOrCreateRagIndexesResponseValue,
)

if typing.TYPE_CHECKING:
    from .crawl_jobs.client import AsyncCrawlJobsClient, CrawlJobsClient
    from .document.client import AsyncDocumentClient, DocumentClient
    from .documents.client import AsyncDocumentsClient, DocumentsClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class KnowledgeBaseClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawKnowledgeBaseClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._documents: typing.Optional[DocumentsClient] = None
        self._crawl_jobs: typing.Optional[CrawlJobsClient] = None
        self._document: typing.Optional[DocumentClient] = None

    @property
    def with_raw_response(self) -> RawKnowledgeBaseClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawKnowledgeBaseClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page_size: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        show_only_owned_documents: typing.Optional[bool] = None,
        created_by_user_id: typing.Optional[str] = None,
        types: typing.Optional[
            typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]
        ] = None,
        parent_folder_id: typing.Optional[str] = None,
        ancestor_folder_id: typing.Optional[str] = None,
        folders_first: typing.Optional[bool] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        sort_by: typing.Optional[KnowledgeBaseSortBy] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetKnowledgeBaseListResponseModel:
        """
        Get a list of available knowledge base documents

        Parameters
        ----------
        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100, defaults to 30.

        search : typing.Optional[str]
            If specified, the endpoint returns only such knowledge base documents whose names start with this string.

        show_only_owned_documents : typing.Optional[bool]
            If set to true, the endpoint will return only documents owned by you (and not shared from somebody else). Deprecated: use created_by_user_id instead.

        created_by_user_id : typing.Optional[str]
            Filter documents by creator user ID. When set, only documents created by this user are returned. Takes precedence over show_only_owned_documents. Use '@me' to refer to the authenticated user.

        types : typing.Optional[typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]]
            If present, the endpoint will return only documents of the given types.

        parent_folder_id : typing.Optional[str]
            If set, the endpoint will return only documents that are direct children of the given folder.

        ancestor_folder_id : typing.Optional[str]
            If set, the endpoint will return only documents that are descendants of the given folder.

        folders_first : typing.Optional[bool]
            Whether folders should be returned first in the list of documents.

        sort_direction : typing.Optional[SortDirection]
            The direction to sort the results

        sort_by : typing.Optional[KnowledgeBaseSortBy]
            The field to sort the results by

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetKnowledgeBaseListResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.list(
            page_size=1,
            search="search",
            show_only_owned_documents=True,
            created_by_user_id="created_by_user_id",
            types=["file"],
            parent_folder_id="parent_folder_id",
            ancestor_folder_id="ancestor_folder_id",
            folders_first=True,
            sort_direction="asc",
            sort_by="name",
            cursor="cursor",
        )
        """
        _response = self._raw_client.list(
            page_size=page_size,
            search=search,
            show_only_owned_documents=show_only_owned_documents,
            created_by_user_id=created_by_user_id,
            types=types,
            parent_folder_id=parent_folder_id,
            ancestor_folder_id=ancestor_folder_id,
            folders_first=folders_first,
            sort_direction=sort_direction,
            sort_by=sort_by,
            cursor=cursor,
            request_options=request_options,
        )
        return _response.data

    def get_or_create_rag_indexes(
        self,
        *,
        items: typing.Sequence[GetOrCreateRagIndexRequestModel],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, KnowledgeBaseGetOrCreateRagIndexesResponseValue]:
        """
        Retrieves and/or creates RAG indexes for multiple knowledge base documents in a single request. Maximum 100 items per request.

        Parameters
        ----------
        items : typing.Sequence[GetOrCreateRagIndexRequestModel]
            List of requested RAG indexes. Minimum 1, maximum 100 items.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, KnowledgeBaseGetOrCreateRagIndexesResponseValue]
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs, GetOrCreateRagIndexRequestModel

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.get_or_create_rag_indexes(
            items=[
                GetOrCreateRagIndexRequestModel(
                    document_id="document_id",
                    create_if_missing=True,
                    model="e5_mistral_7b_instruct",
                )
            ],
        )
        """
        _response = self._raw_client.get_or_create_rag_indexes(items=items, request_options=request_options)
        return _response.data

    def search(
        self,
        *,
        query: str,
        page_size: typing.Optional[int] = None,
        types: typing.Optional[
            typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]
        ] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> KnowledgeBaseContentSearchResponseModel:
        """
        Fuzzy text search over knowledge base document content

        Parameters
        ----------
        query : str
            The search query text

        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100, defaults to 30.

        types : typing.Optional[typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]]
            If present, the endpoint will return only documents of the given types.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        KnowledgeBaseContentSearchResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.search(
            query="query",
            page_size=1,
            types=["file"],
            cursor="cursor",
        )
        """
        _response = self._raw_client.search(
            query=query, page_size=page_size, types=types, cursor=cursor, request_options=request_options
        )
        return _response.data

    @property
    def documents(self):
        if self._documents is None:
            from .documents.client import DocumentsClient  # noqa: E402

            self._documents = DocumentsClient(client_wrapper=self._client_wrapper)
        return self._documents

    @property
    def crawl_jobs(self):
        if self._crawl_jobs is None:
            from .crawl_jobs.client import CrawlJobsClient  # noqa: E402

            self._crawl_jobs = CrawlJobsClient(client_wrapper=self._client_wrapper)
        return self._crawl_jobs

    @property
    def document(self):
        if self._document is None:
            from .document.client import DocumentClient  # noqa: E402

            self._document = DocumentClient(client_wrapper=self._client_wrapper)
        return self._document


class AsyncKnowledgeBaseClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawKnowledgeBaseClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._documents: typing.Optional[AsyncDocumentsClient] = None
        self._crawl_jobs: typing.Optional[AsyncCrawlJobsClient] = None
        self._document: typing.Optional[AsyncDocumentClient] = None

    @property
    def with_raw_response(self) -> AsyncRawKnowledgeBaseClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawKnowledgeBaseClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page_size: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        show_only_owned_documents: typing.Optional[bool] = None,
        created_by_user_id: typing.Optional[str] = None,
        types: typing.Optional[
            typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]
        ] = None,
        parent_folder_id: typing.Optional[str] = None,
        ancestor_folder_id: typing.Optional[str] = None,
        folders_first: typing.Optional[bool] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        sort_by: typing.Optional[KnowledgeBaseSortBy] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetKnowledgeBaseListResponseModel:
        """
        Get a list of available knowledge base documents

        Parameters
        ----------
        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100, defaults to 30.

        search : typing.Optional[str]
            If specified, the endpoint returns only such knowledge base documents whose names start with this string.

        show_only_owned_documents : typing.Optional[bool]
            If set to true, the endpoint will return only documents owned by you (and not shared from somebody else). Deprecated: use created_by_user_id instead.

        created_by_user_id : typing.Optional[str]
            Filter documents by creator user ID. When set, only documents created by this user are returned. Takes precedence over show_only_owned_documents. Use '@me' to refer to the authenticated user.

        types : typing.Optional[typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]]
            If present, the endpoint will return only documents of the given types.

        parent_folder_id : typing.Optional[str]
            If set, the endpoint will return only documents that are direct children of the given folder.

        ancestor_folder_id : typing.Optional[str]
            If set, the endpoint will return only documents that are descendants of the given folder.

        folders_first : typing.Optional[bool]
            Whether folders should be returned first in the list of documents.

        sort_direction : typing.Optional[SortDirection]
            The direction to sort the results

        sort_by : typing.Optional[KnowledgeBaseSortBy]
            The field to sort the results by

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetKnowledgeBaseListResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.list(
                page_size=1,
                search="search",
                show_only_owned_documents=True,
                created_by_user_id="created_by_user_id",
                types=["file"],
                parent_folder_id="parent_folder_id",
                ancestor_folder_id="ancestor_folder_id",
                folders_first=True,
                sort_direction="asc",
                sort_by="name",
                cursor="cursor",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            page_size=page_size,
            search=search,
            show_only_owned_documents=show_only_owned_documents,
            created_by_user_id=created_by_user_id,
            types=types,
            parent_folder_id=parent_folder_id,
            ancestor_folder_id=ancestor_folder_id,
            folders_first=folders_first,
            sort_direction=sort_direction,
            sort_by=sort_by,
            cursor=cursor,
            request_options=request_options,
        )
        return _response.data

    async def get_or_create_rag_indexes(
        self,
        *,
        items: typing.Sequence[GetOrCreateRagIndexRequestModel],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, KnowledgeBaseGetOrCreateRagIndexesResponseValue]:
        """
        Retrieves and/or creates RAG indexes for multiple knowledge base documents in a single request. Maximum 100 items per request.

        Parameters
        ----------
        items : typing.Sequence[GetOrCreateRagIndexRequestModel]
            List of requested RAG indexes. Minimum 1, maximum 100 items.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, KnowledgeBaseGetOrCreateRagIndexesResponseValue]
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs, GetOrCreateRagIndexRequestModel

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.get_or_create_rag_indexes(
                items=[
                    GetOrCreateRagIndexRequestModel(
                        document_id="document_id",
                        create_if_missing=True,
                        model="e5_mistral_7b_instruct",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_or_create_rag_indexes(items=items, request_options=request_options)
        return _response.data

    async def search(
        self,
        *,
        query: str,
        page_size: typing.Optional[int] = None,
        types: typing.Optional[
            typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]
        ] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> KnowledgeBaseContentSearchResponseModel:
        """
        Fuzzy text search over knowledge base document content

        Parameters
        ----------
        query : str
            The search query text

        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100, defaults to 30.

        types : typing.Optional[typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]]
            If present, the endpoint will return only documents of the given types.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        KnowledgeBaseContentSearchResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.search(
                query="query",
                page_size=1,
                types=["file"],
                cursor="cursor",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.search(
            query=query, page_size=page_size, types=types, cursor=cursor, request_options=request_options
        )
        return _response.data

    @property
    def documents(self):
        if self._documents is None:
            from .documents.client import AsyncDocumentsClient  # noqa: E402

            self._documents = AsyncDocumentsClient(client_wrapper=self._client_wrapper)
        return self._documents

    @property
    def crawl_jobs(self):
        if self._crawl_jobs is None:
            from .crawl_jobs.client import AsyncCrawlJobsClient  # noqa: E402

            self._crawl_jobs = AsyncCrawlJobsClient(client_wrapper=self._client_wrapper)
        return self._crawl_jobs

    @property
    def document(self):
        if self._document is None:
            from .document.client import AsyncDocumentClient  # noqa: E402

            self._document = AsyncDocumentClient(client_wrapper=self._client_wrapper)
        return self._document
